import json
class StreamDisposition:
    def __init__(self, disposition):
        self.default = disposition.get("default", 0)
        self.dub = disposition.get("dub", 0)
        self.original = disposition.get("original", 0)
        self.comment = disposition.get("comment", 0)
        self.lyrics = disposition.get("lyrics", 0)
        self.karaoke = disposition.get("karaoke", 0)
        self.forced = disposition.get("forced", 0)
        self.hearing_impaired = disposition.get("hearing_impaired", 0)
        self.visual_impaired = disposition.get("visual_impaired", 0)
        self.clean_effects = disposition.get("clean_effects", 0)
        self.attached_pic = disposition.get("attached_pic", 0)
        self.timed_thumbnails = disposition.get("timed_thumbnails", 0)
        self.non_diegetic = disposition.get("non_diegetic", 0)
        self.captions = disposition.get("captions", 0)
        self.descriptions = disposition.get("descriptions", 0)
        self.metadata = disposition.get("metadata", 0)
        self.dependent = disposition.get("dependent", 0)
        self.still_image = disposition.get("still_image", 0)
        self.multilayer = disposition.get("multilayer", 0)

class StreamTags:
    def __init__(self, tags):
        self.handler_name = tags.get("handler_name", "")
        self.vendor_id = tags.get("vendor_id", "")

class VideoStream:
    def __init__(self, stream):
        self.index = stream.get("index")
        self.codec_name = stream.get("codec_name")
        self.codec_long_name = stream.get("codec_long_name")
        self.profile = stream.get("profile")
        self.codec_type = stream.get("codec_type")
        self.codec_tag_string = stream.get("codec_tag_string")
        self.codec_tag = stream.get("codec_tag")
        self.width = stream.get("width")
        self.height = stream.get("height")
        self.coded_width = stream.get("coded_width")
        self.coded_height = stream.get("coded_height")
        self.has_b_frames = stream.get("has_b_frames")
        self.sample_aspect_ratio = stream.get("sample_aspect_ratio")
        self.display_aspect_ratio = stream.get("display_aspect_ratio")
        self.pix_fmt = stream.get("pix_fmt")
        self.level = stream.get("level")
        self.color_range = stream.get("color_range")
        self.color_space = stream.get("color_space")
        self.color_transfer = stream.get("color_transfer")
        self.color_primaries = stream.get("color_primaries")
        self.chroma_location = stream.get("chroma_location")
        self.field_order = stream.get("field_order")
        self.refs = stream.get("refs")
        self.view_ids_available = stream.get("view_ids_available", "")
        self.view_pos_available = stream.get("view_pos_available", "")
        self.id = stream.get("id")
        self.r_frame_rate = stream.get("r_frame_rate")
        self.avg_frame_rate = stream.get("avg_frame_rate")
        self.time_base = stream.get("time_base")
        self.start_pts = stream.get("start_pts")
        self.start_time = stream.get("start_time")
        self.duration_ts = stream.get("duration_ts")
        self.duration = stream.get("duration")
        self.bit_rate = stream.get("bit_rate")
        self.nb_frames = stream.get("nb_frames")
        self.extradata_size = stream.get("extradata_size")
        self.disposition = StreamDisposition(stream.get("disposition", {}))
        self.tags = StreamTags(stream.get("tags", {}))

class AudioStream:
    def __init__(self, stream):
        self.index = stream.get("index")
        self.codec_name = stream.get("codec_name")
        self.codec_long_name = stream.get("codec_long_name")
        self.profile = stream.get("profile")
        self.codec_type = stream.get("codec_type")
        self.codec_tag_string = stream.get("codec_tag_string")
        self.codec_tag = stream.get("codec_tag")
        self.sample_fmt = stream.get("sample_fmt")
        self.sample_rate = stream.get("sample_rate")
        self.channels = stream.get("channels")
        self.channel_layout = stream.get("channel_layout")
        self.bits_per_sample = stream.get("bits_per_sample")
        self.initial_padding = stream.get("initial_padding")
        self.id = stream.get("id")
        self.r_frame_rate = stream.get("r_frame_rate")
        self.avg_frame_rate = stream.get("avg_frame_rate")
        self.time_base = stream.get("time_base")
        self.start_pts = stream.get("start_pts")
        self.start_time = stream.get("start_time")
        self.duration_ts = stream.get("duration_ts")
        self.duration = stream.get("duration")
        self.bit_rate = stream.get("bit_rate")
        self.nb_frames = stream.get("nb_frames")
        self.extradata_size = stream.get("extradata_size")
        self.disposition = StreamDisposition(stream.get("disposition", {}))
        self.tags = StreamTags(stream.get("tags", {}))

class FormatContext:
    def __init__(self, json_data):
        data = json.loads(json_data)
        streams = data["streams"]
        self.video_streams = []
        self.audio_streams = []
        for stream in streams:
            if stream["codec_type"] == "audio":
                self.audio_streams.append(AudioStream(stream))
            elif stream["codec_type"] == "video": 
                self.video_streams.append(VideoStream(stream))



# server.py
import os
import sys
cur_path=os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, cur_path+"/..")
from typing import List
from mcp.server.fastmcp import FastMCP
import ffmpeg_mcp.cut_video as cut_video



# Create an MCP server
mcp = FastMCP("ffmpeg-mcp")

# Add an addition tool
@mcp.tool()
def find_video_path(root_path, video_name):
    """
    可以查找视频文件路径，查找文件路径，递归查找精确匹配文件名的视频文件路径（支持带或不带扩展名）
    参数：
    root_path - 要搜索的根目录
    video_name - 视频文件名（可以带扩展名，但会忽略扩展名匹配）
    返回：
    首个匹配的视频文件完整路径，找不到时返回空字符串
    """
    VIDEO_EXTS = {'.mp4', '.avi', '.mov', '.mkv', '.flv', '.wmv', '.webm', '.ts'}
    target_stem, target_ext = os.path.splitext(video_name)
    if target_ext.lower() not in VIDEO_EXTS:
        target_stem = f"{target_stem}{target_ext}"
        target_ext = ""

    for root, dirs, files in os.walk(root_path):
        for file in files:
            stem, ext = os.path.splitext(file)
            if stem.lower() == target_stem.lower():
                if (not target_ext or ext.lower() in VIDEO_EXTS):
                    return os.path.join(root, file)
    return ""

@mcp.tool()
def clip_video(video_path, start=None, end=None,duration = None, output_path=None,time_out=300):
    """
    智能视频剪辑函数
    
    参数：
    video_path : str - 源视频文件路径
    start : int/float/str - 开始时间（支持秒数、MM:SS、HH:MM:SS格式,默认为视频开头,如果不传该参数，或者该参数为负数，从视频结尾往前剪辑）
    end : int/float/str - 结束时间（同上，默认为视频结尾）
    duration:  int/float/str - 裁剪时长，end和duration必须有一个
    output_path: str - 裁剪后视频输出路径，如果不传入，会有一个默认的输出路径
    time_out: int - 命令行执行超时时间，默认为300s
    返回：
    error - 错误码
    str - ffmpeg执行过程中所有日志
    str - 生成的剪辑文件路径
    示例：
    clip_video("input.mp4", "00:01:30", "02:30")
    """
    return cut_video.clip_video_ffmpeg(video_path,start=start,end=end,duration=duration, output_path=output_path,time_out=time_out)   

@mcp.tool()
def concat_videos(input_files: List[str], output_path: str = None, 
                      fast: bool = True):
    """
    使用FFmpeg拼接多个视频文件
    
    参数:
    input_files (List[str]): 输入视频文件路径列表
    output_path (str): 合并后的输出文件路径,如果不传入，会一个默认的输出路径
    fast (bool): 拼接方法，可选值："True"（默认，要求所有视频必须具有相同的编码格式、分辨率、帧率等参数）| "False(当不确定合并的视频编码格式、分辨率、帧率等参数是否相同的情况下，这个参数应该是False)"
    
    返回:
    执行日志
    
    注意:
    1. 当fast=True时，要求所有视频必须具有相同的编码格式、分辨率、帧率等参数
    2. 推荐视频文件使用相同编码参数，避免拼接失败
    3. 输出文件格式由output_path后缀决定（如.mp4/.mkv）
    """
    return cut_video.concat_videos(input_files,output_path,fast)

@mcp.tool()
def get_video_info(video_path: str):
    """
    获取视频信息，包括时长，帧率，codec等
    
    参数:
    video_path (str): 输入视频文件路径
    返回:
    视频详细信息
    """
    return cut_video.get_video_info(video_path)

@mcp.tool()
def play_video(video_path, speed = 1, loop = 1):
    """
    使用 ffplay 播放视频文件，支持mkv,mp4,mov,avi,3gp等等

    参数：
    video_path(str) - 视频文件的路径。
    speed(float) - 浮点型,播放速率,建议0.5-2之间。
    loop(int) - 整形,是否循环播放,1:不循环,播放后就退出,0: 循环播放。
    """
    return cut_video.video_play(video_path,speed=speed,loop=loop)


@mcp.tool()
def overlay_video(background_video, overlay_video, output_path: str = None, position: int = 1,  dx = 0, dy = 0):
    """
    两个视频叠加，注意不是拼接长度，而是画中画效果

    参数：
    background_video(str) - 背景视频文件的路径。
    overlay_video(str) - 前景视频文件路径。
    output_path(str) - 输出路径
    position(enum) - 相对位置，TopLeft=1: 左上角,TopCenter=2: 上居中, TopRight=3: 右上角 RightCenter=4: 右居中 BottomRight=5: 右下角 BottomCenter=6: 下居中 BottomLeft=7: 左下角 LeftCenter=8: 左居中 Center=9: 居中
    dx(int) - 整形,前景视频坐标x偏移值
    dy(int) - 整形,前景视频坐标y偏移值
    """
    return cut_video.overlay_video(background_video, overlay_video, output_path,position, dx, dy)
       
@mcp.tool()   
def scale_video(video_path, width, height,output_path: str = None):
    """
    视频缩放

    参数：
    width(int) - 目标宽度。
    height(int) - 目标高度。
    output_path(str) - 输出路径
    """ 
    return cut_video.scale_video(video_path, width, height, output_path)

@mcp.tool()   
def extract_frames_from_video(video_path,fps=0, output_folder=None, format=0, total_frames=0):
    """
    提取视频中的图像。

    参数：
    video_path(str) - 视频路径。
    fps(int) - 每多少秒抽一帧，如果传0，代表全部都抽,传1，代表每一秒抽1帧。
    output_folder(str) - 把图片输出到哪个目录
    format(int) - 抽取的图片格式，0：代表png 1:jpg 2:webp
    total_frames(int) - 最多抽取多少张，0代表不限制
    """ 
    return cut_video.extract_frames_from_video(video_path, fps, output_folder, format, total_frames)

def main():
    print("Server running")
    mcp.run(transport='stdio')
if __name__ == "__main__":
    main()


import subprocess
import shlex
import sys
import threading
import os
import platform
import ffmpeg_mcp.utils as utils
import ffmpeg_mcp.typedef as typedef
def check_os_architecture():
    # 获取当前操作系统
    system = platform.system()
    # 获取处理器架构
    machine = platform.machine()
    return system, machine
        
def run_command(command, timeout=300):
    """
    运行FFmpeg命令并捕获相关信息。

    参数:
        command (str): 要执行的FFmpeg命令行字符串。
        timeout (int): 命令执行的超时时间（以秒为单位），默认300秒。

    返回:
        tuple: 包含以下元素的元组：
            - return_code (int): 命令执行后的返回状态码。
            - output_log (str): 命令执行过程中的标准输出日志。
    """
    logs = []
    return_code = 0
    append_msg = ""
    def read_output(proc):
        try:
            while True:
                line = proc.stdout.readline()
                if not line and proc.poll() is not None:
                    break
                if line:
                    logs.append(line)
        except ValueError:
            # 处理文件关闭时的异常
            pass
    try:
        # 使用subprocess.run执行FFmpeg命令
        args = shlex.split(command, posix=(sys.platform != 'win32'))
        proc = subprocess.Popen(
            args,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
            bufsize=1,
            encoding='utf-8',
            errors='replace'
        )
        thread = threading.Thread(target=read_output, args=(proc,))
        thread.start()
        return_code = proc.wait(timeout=timeout)
    except subprocess.TimeoutExpired:
        print("command timed out")
        return_code = -1
        append_msg = "Timeout expired"
    except Exception as e:
        return_code = -1
        append_msg = f"An error occurred: {e}"
        # 捕获所有其他异常
    finally:
        # 确保清理资源
        if 'proc' in globals() and proc and proc.poll() is None:  # 如果进程仍在运行
            proc.kill()
        if 'thread' in globals() and thread:
            thread.join()  #
        logs.append(append_msg)
        return return_code, '\n'.join(logs), append_msg
    
def is_file_and_exists(file_path):
    return os.path.isfile(file_path) and os.path.exists(file_path)

def command_dir():
    system,machine = check_os_architecture()
    current_work_dir = os.path.dirname(__file__)
    os.chdir(f"{current_work_dir}/bin")
    if system == "Darwin":
        if machine == "x86_64":
            dir = f"{current_work_dir}/bin/ffmpeg-macos-x86_64"
            url = "https://gitee.com/littlecodergitxxx/ffmpeg-macos-x86_64.git"
        if machine == "arm64":
            dir = f"{current_work_dir}/bin/ffmpeg-macos-arm64"
            url = "https://gitee.com/littlecodergitxxx/ffmpeg-macos-arm64.git"
        if not is_file_and_exists(f"{dir}/ffmpeg"):
            cmd = f"rm -rf {dir}"
            code,_,_ = run_command(cmd)
            cmd = f"git clone {url}"
            code,_,_ = run_command(cmd)
            if code == 0:
                os.chdir(dir)
            else:
                return None
            cmd = f"git checkout v0.1"
            code,_,_= run_command(cmd)
            if code == 0:
                utils.unzip_to_current_directory(f"ffmpeg.zip")
            else:
                return None
            cmd = f"chmod 777 {dir}/ffmpeg"
            code,_,_= run_command(cmd)
            cmd = f"chmod 777 {dir}/ffprobe"
            code,_,_= run_command(cmd)
            cmd = f"chmod 777 {dir}/ffplay"
            code,_,_= run_command(cmd)
        return dir
    return None

def run_ffmpeg(cmd, timeout=300):
    cmd_dir = command_dir()
    if cmd_dir is None:
        return -1, "Not Support Platform"
    cmd = f"{cmd_dir}/ffmpeg {cmd}"
    logs = []
    logs.append(cmd)
    code, log, append_msg = run_command(cmd,timeout)
    logs.append(log)
    logs.append(append_msg)
    return code, '\n'.join(logs)

def run_ffprobe(cmd, timeout = 60):
    cmd_dir = command_dir()
    if cmd_dir is None:
        return -1, "Not Support Platform"
    cmd = f"{cmd_dir}/ffprobe {cmd}"
    code, log, append_msg = run_command(cmd,timeout)
    logs = []
    if (code != 0):
        logs.append(cmd)
        logs.append(log)
        logs.append(append_msg)
        return code,cmd,'\n'.join(logs)
    return code, cmd, log

def run_ffplay(cmd, timeout = 60):
    cmd_dir = command_dir()
    if cmd_dir is None:
        return -1, "Not Support Platform"
    cmd = f"{cmd_dir}/ffplay {cmd}"
    code, log, append_msg = run_command(cmd,timeout)
    logs = []
    if (code != 0):
        logs.append(cmd)
        logs.append(log)
        logs.append(append_msg)
        return code,cmd,'\n'.join(logs)
    return code, cmd, log
    
def media_format_ctx(path):
    cmd = f" -show_streams -of json -v error -i {path}"
    code, cmd, log = run_ffprobe(cmd)
    if (code == 0):
        return typedef.FormatContext(log)
    return None